from typing import List

from bluer_options.terminal import show_usage


def help_list_targets(
    tokens: List[str],
    mono: bool,
) -> str:
    args = [
        "[--delim +]",
    ]

    return show_usage(
        [
            "vanwatch",
            "list_targets",
        ]
        + args,
        "list targets.",
        mono=mono,
    )



from tqdm import tqdm
import geopandas as gpd
from shapely.geometry import Point

from blueness import module
from bluer_objects import objects
from bluer_objects.file import load_json
from bluer_objects.metadata import post_to_object
from bluer_geo.file import save_geojson

from vancouver_watching import NAME
from vancouver_watching.QGIS import label_of_camera
from vancouver_watching.logger import logger

NAME = module.name(__file__, NAME)


# https://511on.ca/help/endpoint/cameras
def toronto(
    object_name: str,
    prefix: str,
    count: int = -1,
) -> bool:
    logger.info(
        "{}.discover({}{}) -> {}".format(
            NAME,
            prefix,
            "" if count == -1 else f"count={count}",
            object_name,
        )
    )

    success, list_of_cameras_raw = load_json(
        objects.path_of(
            object_name=object_name,
            filename="detections.json",
        )
    )
    if not success:
        return False

    camera_count = 0
    records = []
    for row in tqdm(list_of_cameras_raw):
        list_of_cameras_ = [view["Url"] for view in row["Views"]]
        camera_count += len(list_of_cameras_)

        records.append(
            {
                "cameras": ",".join(list_of_cameras_),
                "geometry": Point(row["Longitude"], row["Latitude"]),
                "label": label_of_camera(
                    "#",
                    row["Location"],
                    list_of_cameras_,
                ),
                "mapid": row["SourceId"],
            }
        )

        if count != -1 and len(records) >= count:
            break

    gdf = gpd.GeoDataFrame(
        records,
        geometry="geometry",
        crs="EPSG:4326",
    )

    logger.info(
        "found {} camera(s) in {} location(s).".format(
            camera_count,
            len(gdf),
        )
    )

    if not save_geojson(
        objects.path_of(
            object_name=object_name,
            filename="detections.geojson",
        ),
        gdf,
    ):
        return False

    return post_to_object(
        object_name,
        "discovery",
        {
            "target": "toronto",
            "cameras": camera_count,
            "locations": len(gdf),
        },
    )



import argparse

from blueness import module
from blueness.argparse.generic import sys_exit

from vancouver_watching import NAME
from vancouver_watching.discover.targets import list_of_targets
from vancouver_watching.discover.vancouver import discover as discover_vancouver
from vancouver_watching.discover.toronto import toronto as discover_toronto
from vancouver_watching.discover.testing import test as test_discovery
from vancouver_watching.logger import logger

NAME = module.name(__file__, NAME)

parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help=" | ".join(
        sorted(
            [
                "list_of_targets",
                "test",
            ]
            + list_of_targets()
        )
    ),
)
parser.add_argument(
    "--object_name",
    type=str,
    default="",
)
parser.add_argument(
    "--prefix",
    type=str,
    default="",
)
parser.add_argument(
    "--delim",
    type=str,
    default="+",
)
parser.add_argument(
    "--count",
    type=int,
    default="-1",
    help="-1 for all",
)
args = parser.parse_args()

delim = " " if args.delim == "space" else args.delim

success = False
if args.task == "list_targets":
    print(delim.join(list_of_targets()))
    success = True
elif args.task == "test":
    success = test_discovery(
        object_name=args.object_name,
    )
elif args.task == "vancouver":
    success = discover_vancouver(
        object_name=args.object_name,
        prefix=args.prefix,
        count=args.count,
    )
elif args.task == "toronto":
    success = discover_toronto(
        object_name=args.object_name,
        prefix=args.prefix,
        count=args.count,
    )
else:
    success = None

sys_exit(logger, NAME, args.task, success)



NAME = "vancouver_watching"

ICON = "🌈"

DESCRIPTION = f"{ICON} Vancouver Watching with AI."

VERSION = "4.34.1"

REPO_NAME = "vancouver-watching"

MARQUEE = "https://github.com/kamangir/assets/blob/main/vanwatch/QGIS.png?raw=true"

ALIAS = "vanwatch"


def fullname() -> str:
    return f"{NAME}-{VERSION}"



from typing import List


def label_of_camera(
    location_url: str,
    location_name: str,
    list_of_cameras: List[str],
) -> str:
    return '<a href="{}">{}</a><br/> {}'.format(
        location_url,
        location_name,
        f'<img src="{list_of_cameras[0]}">' if list_of_cameras else "camera not found.",
    )



from blueness import module
from bluer_objects import objects, file
from bluer_geo.file import load_geodataframe

from vancouver_watching import NAME
from vancouver_watching.logger import logger

NAME = module.name(__file__, NAME)


def test(object_name: str) -> bool:
    filename = objects.path_of(
        object_name=object_name,
        filename="detections.geojson",
    )

    logger.info(f"{NAME}.test({object_name}): {filename}")

    success, gdf = load_geodataframe(filename)
    if not success:
        logger.error("can not load detections.geojson!")
        return False
    logger.info("✅ loaded detections.geojson.")

    for column_name in [
        "cameras",
        "label",
        "mapid",
    ]:
        if column_name not in gdf.columns:
            logger.error(f"{column_name} not in detections.geojson!")
            return False
        logger.info(f"✅ {column_name} in detections.geojson.")

    return True



import os
from typing import List

from bluer_objects import file


def list_of_targets() -> List[str]:
    return sorted(
        [
            file.name(filename)
            for filename in file.list_of(
                os.path.join(
                    file.path(__file__),
                    "../.abcli/discovery/*.sh",
                )
            )
        ]
    )



import os

from bluer_objects import file, README
from bluer_options.help.functions import get_help

from vancouver_watching import NAME, VERSION, ICON, REPO_NAME
from vancouver_watching.discover.targets import list_of_targets
from vancouver_watching.help.functions import help_functions


items = README.Items(
    [
        {
            "name": target.title(),
            "marquee": f"https://github.com/kamangir/assets/blob/main/vanwatch-ingest-{target}/vanwatch-ingest-{target}.gif?raw=true",
            "url": f"./vancouver_watching/docs/{target}.md",
        }
        for target in list_of_targets()
    ]
    + [
        {
            "name": "time-series",
            "marquee": "https://github.com/kamangir/assets/blob/main/vanwatch/2024-01-06-20-39-46-73614-QGIS.gif?raw=true",
            "url": "https://github.com/kamangir/assets/blob/main/vanwatch/2024-01-06-20-39-46-73614-QGIS.gif",
        },
    ]
)


def build():
    return all(
        README.build(
            items=readme.get("items", []),
            path=os.path.join(file.path(__file__), readme["path"]),
            ICON=ICON,
            NAME=NAME,
            VERSION=VERSION,
            REPO_NAME=REPO_NAME,
            help_function=lambda tokens: get_help(
                tokens,
                help_functions,
                mono=True,
            ),
        )
        for readme in [
            {"items": items, "path": ".."},
        ]
        + [{"path": f"./docs/{target}.md"} for target in list_of_targets()]
    )



from bluer_options.env import load_config, load_env, get_env

load_env(__name__)
load_config(__name__)

ULTRALYTICS_API_KEY = get_env("ULTRALYTICS_API_KEY")

VANWATCH_DEFAULT_MODEL = get_env("VANWATCH_DEFAULT_MODEL")

VANWATCH_TEST_OBJECT = get_env("VANWATCH_TEST_OBJECT")

VANWATCH_QGIS_TEMPLATE = get_env("VANWATCH_QGIS_TEMPLATE")
